@@ -1,18 +1,24 @@
+use std::fs;
 use std::io;
 use std::io::prelude::*;
 use std::io::SeekFrom;
 use std::str;
 
+use log::warn;
+
 use lazy_static::lazy_static;
 use regex::Regex;
 
 use crate::peaks::{CentroidPeak, PeakCollection, PeakSet};
 use crate::spectrum::{
-    scan_properties, CentroidSpectrum, Precursor, SelectedIon, SpectrumDescription,
+    scan_properties, CentroidSpectrum, Precursor, RawSpectrum, SelectedIon, Spectrum,
+    SpectrumDescription,
 };
 
 use super::offset_index::OffsetIndex;
-use super::traits::{RandomAccessScanIterator, ScanAccessError, ScanSource};
+use super::traits::{
+    MZFileReader, RandomAccessScanIterator, ScanAccessError, ScanSource, SeekRead,
+};
 
 #[derive(PartialEq, Debug)]
 pub enum MGFParserState {
@@ -34,9 +40,58 @@ pub enum MGFError {
     IOError,
 }
 
+#[derive(Debug, Clone)]
+struct SpectrumBuilder {
+    pub peaks: PeakSet,
+    pub description: SpectrumDescription,
+}
+
+impl Default for SpectrumBuilder {
+    fn default() -> SpectrumBuilder {
+        SpectrumBuilder {
+            peaks: PeakSet::default(),
+            description: SpectrumDescription {
+                ms_level: 2,
+                signal_continuity: scan_properties::SignalContinuity::Centroid,
+                polarity: scan_properties::ScanPolarity::Unknown,
+                ..Default::default()
+            },
+        }
+    }
+}
+
+impl SpectrumBuilder {
+    pub fn into_spectrum(self, spectrum: &mut Spectrum) {
+        spectrum.description = self.description;
+        spectrum.peaks = Some(self.peaks);
+    }
+}
+
+impl From<SpectrumBuilder> for Spectrum {
+    fn from(builder: SpectrumBuilder) -> Spectrum {
+        let mut spectrum = Spectrum::default();
+        builder.into_spectrum(&mut spectrum);
+        spectrum
+    }
+}
+
+macro_rules! impl_from_spectrum_builder_for_spec {
+    ($($t:ty), +) => {$(
+        impl From<SpectrumBuilder> for $t {
+            fn from(builder: SpectrumBuilder) -> $t {
+                let spec: Spectrum = builder.into();
+                spec.into()
+            }
+        }
+    )+};
+}
+
+impl_from_spectrum_builder_for_spec!(CentroidSpectrum, RawSpectrum);
+
 /// An MGF (Mascot Generic Format) file parser that supports iteration and random access.
-/// The parser produces [`CentroidSpectrum`] instances that represent the pre-processed
-/// nature of this type of file's data.
+/// The parser produces [`Spectrum`] instances. These may be converted directly into [`CentroidSpectrum`]
+/// instances using [`Spectrum.into_centroid`] or the [`From`] trait which better represent
+/// the nature of this preprocessed data type.
 pub struct MGFReader<R: io::Read> {
     pub handle: io::BufReader<R>,
     pub state: MGFParserState,
@@ -172,31 +227,17 @@ impl<R: io::Read> MGFReader<R> {
         true
     }
 
-    /// Make a new, empty scan with the appropriate default values set
-    /// for this type of file.
-    pub fn new_scan(&self) -> CentroidSpectrum {
-        let description: SpectrumDescription = SpectrumDescription {
-            ms_level: 2,
-            signal_continuity: scan_properties::SignalContinuity::Centroid,
-            polarity: scan_properties::ScanPolarity::Unknown,
-            ..Default::default()
-        };
-
-        let peaks: PeakSet = PeakSet::empty();
-        CentroidSpectrum { description, peaks }
-    }
-
     fn read_line(&mut self, buffer: &mut String) -> io::Result<usize> {
         self.handle.read_line(buffer)
     }
 
     /// Read the next spectrum from the file, if there is one.
-    pub fn read_next(&mut self) -> Option<CentroidSpectrum> {
-        let mut scan = self.new_scan();
-        match self.read_into(&mut scan) {
+    pub fn read_next(&mut self) -> Option<Spectrum> {
+        let mut builder = SpectrumBuilder::default();
+        match self._parse_into(&mut builder) {
             Ok(offset) => {
                 if offset > 0 {
-                    Some(scan)
+                    Some(builder.into())
                 } else {
                     None
                 }
@@ -209,7 +250,7 @@ impl<R: io::Read> MGFReader<R> {
     }
 
     /// Read the next spectrum's contents directly into the passed struct.
-    pub fn read_into(&mut self, spectrum: &mut CentroidSpectrum) -> Result<usize, MGFError> {
+    fn _parse_into(&mut self, spectrum: &mut SpectrumBuilder) -> Result<usize, MGFError> {
         let mut buffer = String::new();
         let mut work = true;
         let mut offset: usize = 0;
@@ -256,7 +297,16 @@ impl<R: io::Read> MGFReader<R> {
         }
         Ok(offset)
     }
-
+    pub fn read_into(&mut self, spectrum: &mut Spectrum) -> Result<usize, MGFError> {
+        let mut accumulator = SpectrumBuilder::default();
+        match self._parse_into(&mut accumulator) {
+            Ok(sz) => {
+                accumulator.into_spectrum(spectrum);
+                Ok(sz)
+            }
+            Err(err) => Err(err),
+        }
+    }
     /// Create a new, unindexed MGF parser
     pub fn new(file: R) -> MGFReader<R> {
         let handle = io::BufReader::with_capacity(500, file);
@@ -271,7 +321,7 @@ impl<R: io::Read> MGFReader<R> {
 }
 
 impl<R: io::Read> Iterator for MGFReader<R> {
-    type Item = CentroidSpectrum;
+    type Item = Spectrum;
 
     /// Read the next spectrum from the file.
     fn next(&mut self) -> Option<Self::Item> {
@@ -279,7 +329,7 @@ impl<R: io::Read> Iterator for MGFReader<R> {
     }
 }
 
-impl<R: io::Read + io::Seek> MGFReader<R> {
+impl<R: SeekRead> MGFReader<R> {
     /// Construct a new MGFReader and build an offset index
     /// using [`Self::build_index`]
     pub fn new_indexed(file: R) -> MGFReader<R> {
@@ -292,7 +342,6 @@ impl<R: io::Read + io::Seek> MGFReader<R> {
         self.handle.seek(pos)
     }
 
-
     /// Builds an offset index to each `BEGIN IONS` line
     /// by doing a fast pre-scan of the text file.
     pub fn build_index(&mut self) -> u64 {
@@ -338,13 +387,17 @@ impl<R: io::Read + io::Seek> MGFReader<R> {
         }
         self.seek(SeekFrom::Start(start))
             .expect("Failed to restore location");
+        self.index.init = true;
+        if self.index.len() == 0 {
+            warn!("An index was built but no entries were found")
+        }
         offset
     }
 }
 
-impl<R: io::Read + io::Seek> ScanSource<CentroidSpectrum> for MGFReader<R> {
+impl<R: SeekRead> ScanSource<Spectrum> for MGFReader<R> {
     /// Retrieve a spectrum by it's native ID
-    fn get_spectrum_by_id(&mut self, id: &str) -> Option<CentroidSpectrum> {
+    fn get_spectrum_by_id(&mut self, id: &str) -> Option<Spectrum> {
         let offset_ref = self.index.get(id);
         let offset = offset_ref.expect("Failed to retrieve offset");
         let start = self
@@ -360,7 +413,7 @@ impl<R: io::Read + io::Seek> ScanSource<CentroidSpectrum> for MGFReader<R> {
     }
 
     /// Retrieve a spectrum by it's integer index
-    fn get_spectrum_by_index(&mut self, index: usize) -> Option<CentroidSpectrum> {
+    fn get_spectrum_by_index(&mut self, index: usize) -> Option<Spectrum> {
         let (_id, offset) = self.index.get_index(index)?;
         let byte_offset = offset;
         let start = self
@@ -382,16 +435,23 @@ impl<R: io::Read + io::Seek> ScanSource<CentroidSpectrum> for MGFReader<R> {
     }
 
     fn get_index(&self) -> &OffsetIndex {
+        if !self.index.init {
+            warn!("Attempting to use an uninitialized offset index on MGFReader")
+        }
         &self.index
     }
+
+    fn set_index(&mut self, index: OffsetIndex) {
+        self.index = index;
+    }
 }
 
-impl<R: io::Read + io::Seek> RandomAccessScanIterator<CentroidSpectrum> for MGFReader<R> {
+impl<R: SeekRead> RandomAccessScanIterator<Spectrum> for MGFReader<R> {
     fn start_from_id(&mut self, id: &str) -> Result<&Self, ScanAccessError> {
         match self._offset_of_id(id) {
             Some(offset) => match self.seek(SeekFrom::Start(offset)) {
                 Ok(_) => Ok(self),
-                Err(err) => Err(ScanAccessError::IOError(err)),
+                Err(err) => Err(ScanAccessError::IOError(Some(err))),
             },
             None => Err(ScanAccessError::ScanNotFound),
         }
@@ -401,7 +461,7 @@ impl<R: io::Read + io::Seek> RandomAccessScanIterator<CentroidSpectrum> for MGFR
         match self._offset_of_index(index) {
             Some(offset) => match self.seek(SeekFrom::Start(offset)) {
                 Ok(_) => Ok(self),
-                Err(err) => Err(ScanAccessError::IOError(err)),
+                Err(err) => Err(ScanAccessError::IOError(Some(err))),
             },
             None => Err(ScanAccessError::ScanNotFound),
         }
@@ -411,61 +471,23 @@ impl<R: io::Read + io::Seek> RandomAccessScanIterator<CentroidSpectrum> for MGFR
         match self._offset_of_time(time) {
             Some(offset) => match self.seek(SeekFrom::Start(offset)) {
                 Ok(_) => Ok(self),
-                Err(err) => Err(ScanAccessError::IOError(err)),
+                Err(err) => Err(ScanAccessError::IOError(Some(err))),
             },
             None => Err(ScanAccessError::ScanNotFound),
         }
     }
 }
 
-pub struct MGFIterator<'lifespan, R: io::Read + io::Seek> {
-    source: &'lifespan mut MGFReader<R>,
-    index: usize,
-    back_index: usize,
-}
+impl MZFileReader<Spectrum> for MGFReader<fs::File> {
+    fn open_file(source: fs::File) -> Self {
+        Self::new(source)
+    }
 
-impl<'lifespan, R: io::Read + io::Seek> MGFIterator<'lifespan, R> {
-    pub fn new(source: &'lifespan mut MGFReader<R>) -> MGFIterator<R>{
-        MGFIterator {
-            source,
-            index: 0,
-            back_index: 0,
-        }
+    fn construct_index_from_stream(&mut self) -> u64 {
+        self.build_index()
     }
 }
 
-impl<'lifespan, R: io::Read + io::Seek> Iterator for MGFIterator<'lifespan, R> {
-    type Item = CentroidSpectrum;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        if self.index + self.back_index >= self.len() {
-            return None;
-        }
-        let result = self.source.get_spectrum_by_index(self.index);
-        self.index += 1;
-        result
-    }
-}
-
-impl<'lifespan, R: io::Read + io::Seek> ExactSizeIterator for MGFIterator<'lifespan, R> {
-    fn len(&self) -> usize {
-        self.source.len()
-    }
-}
-
-impl<'lifespan, R: io::Read + io::Seek> DoubleEndedIterator for MGFIterator<'lifespan, R> {
-    fn next_back(&mut self) -> Option<Self::Item> {
-        if self.index + self.back_index >= self.len() {
-            return None;
-        };
-        let i = self.len() - (self.back_index + 1);
-        let result = self.source.get_spectrum_by_index(i);
-        self.back_index += 1;
-        result
-    }
-}
-
-
 #[cfg(test)]
 mod test {
     use super::*;
