@@ -1,21 +1,26 @@
 //! Implements a parser for the PSI-MS mzML and indexedmzML XML file formats
 //! for representing raw and processed mass spectra.
 
+use std::fs;
 use std::io;
 use std::io::{BufReader, Read, Seek, SeekFrom};
 
+use log::warn;
+
 use quick_xml::events::{BytesEnd, BytesStart, BytesText, Event};
 use quick_xml::Error as XMLError;
 use quick_xml::Reader;
 
 use super::offset_index::OffsetIndex;
-use super::traits::{RandomAccessScanIterator, ScanAccessError, ScanSource};
+use super::traits::{
+    MZFileReader, RandomAccessScanIterator, ScanAccessError, ScanSource, SeekRead,
+};
 use crate::params::{Param, ParamList};
 use crate::spectrum::scan_properties::*;
 use crate::spectrum::signal::{
     ArrayType, BinaryArrayMap, BinaryCompressionType, BinaryDataArrayType, DataArray,
 };
-use crate::spectrum::RawSpectrum;
+use crate::spectrum::{CentroidSpectrum, RawSpectrum, Spectrum};
 
 pub type Bytes = Vec<u8>;
 
@@ -134,7 +139,7 @@ impl MzMLSpectrumBuilder {
         self.polarity = ScanPolarity::Unknown;
     }
 
-    pub fn _to_spectrum(&self, spectrum: &mut RawSpectrum) {
+    pub fn _to_spectrum(&self, spectrum: &mut Spectrum) {
         let description = &mut spectrum.description;
 
         description.id = self.scan_id.clone();
@@ -151,10 +156,10 @@ impl MzMLSpectrumBuilder {
             description.precursor = None;
         }
 
-        spectrum.arrays = self.arrays.clone();
+        spectrum.arrays = Some(self.arrays.clone());
     }
 
-    pub fn into_spectrum(self, spectrum: &mut RawSpectrum) {
+    pub fn into_spectrum(self, spectrum: &mut Spectrum) {
         let description = &mut spectrum.description;
 
         description.id = self.scan_id;
@@ -171,7 +176,7 @@ impl MzMLSpectrumBuilder {
             description.precursor = None;
         }
 
-        spectrum.arrays = self.arrays;
+        spectrum.arrays = Some(self.arrays);
     }
 
     fn handle_param<B: io::BufRead>(
@@ -650,8 +655,32 @@ impl MzMLSpectrumBuilder {
     }
 }
 
+impl Into<CentroidSpectrum> for MzMLSpectrumBuilder {
+    fn into(self) -> CentroidSpectrum {
+        let mut spec = Spectrum::default();
+        self.into_spectrum(&mut spec);
+        spec.into()
+    }
+}
+
+impl Into<Spectrum> for MzMLSpectrumBuilder {
+    fn into(self) -> Spectrum {
+        let mut spec = Spectrum::default();
+        self.into_spectrum(&mut spec);
+        spec
+    }
+}
+
+impl Into<RawSpectrum> for MzMLSpectrumBuilder {
+    fn into(self) -> RawSpectrum {
+        let mut spec = Spectrum::default();
+        self.into_spectrum(&mut spec);
+        spec.into()
+    }
+}
+
 /// An mzML parser that supports iteration and random access. The parser produces
-/// [`RawSpectrum`] instances, which may be converted up to [`Spectrum`](crate::spectrum::spectrum::Spectrum)
+/// [`Spectrum`] instances, which may be converted up to [`Spectrum`](crate::spectrum::spectrum::Spectrum)
 /// or forward to [`CentroidSpectrum`](crate::spectrum::CentroidSpectrum) as is appropriate to the data.
 ///
 /// When the readable stream the parser is wrapped around supports [`io::Seek`],
@@ -773,10 +802,10 @@ impl<R: Read> MzMLReader<R> {
         }
     }
 
-    /// Populate a new [`RawSpectrum`] in-place on the next available spectrum data.
+    /// Populate a new [`Spectrum`] in-place on the next available spectrum data.
     /// This allocates memory to build the spectrum's attributes but then moves it
     /// into `spectrum` rather than copying it.
-    pub fn read_into(&mut self, spectrum: &mut RawSpectrum) -> Result<usize, MzMLParserError> {
+    pub fn read_into(&mut self, spectrum: &mut Spectrum) -> Result<usize, MzMLParserError> {
         let mut accumulator = MzMLSpectrumBuilder::new();
         if self.state == MzMLParserState::SpectrumDone {
             self.state = MzMLParserState::Resume;
@@ -791,30 +820,29 @@ impl<R: Read> MzMLReader<R> {
     }
 
     /// Read the next spectrum directly. Used to implement iteration.
-    pub fn read_next(&mut self) -> Option<RawSpectrum> {
-        let mut spectrum = RawSpectrum::default();
+    pub fn read_next(&mut self) -> Option<Spectrum> {
+        let mut spectrum = Spectrum::default();
         match self.read_into(&mut spectrum) {
-            Ok(_sz) => Some(spectrum),
+            Ok(_sz) => Some(spectrum.into()),
             Err(_err) => None,
         }
     }
 }
 
-/// [`MzMLReader`] instances are [`Iterator`]s over [`RawSpectrum`]
+/// [`MzMLReader`] instances are [`Iterator`]s over [`Spectrum`]
 impl<R: io::Read> Iterator for MzMLReader<R> {
-    type Item = RawSpectrum;
+    type Item = Spectrum;
 
     fn next(&mut self) -> Option<Self::Item> {
         self.read_next()
     }
 }
 
-
 /// They can also be used to fetch specific spectra by ID, index, or start
 /// time when the underlying file stream supports [`io::Seek`].
-impl<R: io::Read + io::Seek> ScanSource<RawSpectrum> for MzMLReader<R> {
+impl<R: SeekRead> ScanSource<Spectrum> for MzMLReader<R> {
     /// Retrieve a spectrum by it's native ID
-    fn get_spectrum_by_id(&mut self, id: &str) -> Option<RawSpectrum> {
+    fn get_spectrum_by_id(&mut self, id: &str) -> Option<Spectrum> {
         let offset_ref = self.index.get(id);
         let offset = offset_ref.expect("Failed to retrieve offset");
         let start = self
@@ -830,7 +858,7 @@ impl<R: io::Read + io::Seek> ScanSource<RawSpectrum> for MzMLReader<R> {
     }
 
     /// Retrieve a spectrum by it's integer index
-    fn get_spectrum_by_index(&mut self, index: usize) -> Option<RawSpectrum> {
+    fn get_spectrum_by_index(&mut self, index: usize) -> Option<Spectrum> {
         let (_id, offset) = self.index.get_index(index)?;
         let byte_offset = offset;
         let start = self
@@ -852,18 +880,25 @@ impl<R: io::Read + io::Seek> ScanSource<RawSpectrum> for MzMLReader<R> {
     }
 
     fn get_index(&self) -> &OffsetIndex {
+        if !self.index.init {
+            warn!("Attempting to use an uninitialized offset index on MzMLReader")
+        }
         &self.index
     }
+
+    fn set_index(&mut self, index: OffsetIndex) {
+        self.index = index
+    }
 }
 
 /// The iterator can also be updated to move to a different location in the
 /// stream efficiently.
-impl<R: io::Read + io::Seek> RandomAccessScanIterator<RawSpectrum> for MzMLReader<R> {
+impl<R: SeekRead> RandomAccessScanIterator<Spectrum> for MzMLReader<R> {
     fn start_from_id(&mut self, id: &str) -> Result<&Self, ScanAccessError> {
         match self._offset_of_id(id) {
             Some(offset) => match self.seek(SeekFrom::Start(offset)) {
                 Ok(_) => Ok(self),
-                Err(err) => Err(ScanAccessError::IOError(err)),
+                Err(err) => Err(ScanAccessError::IOError(Some(err))),
             },
             None => Err(ScanAccessError::ScanNotFound),
         }
@@ -873,7 +908,7 @@ impl<R: io::Read + io::Seek> RandomAccessScanIterator<RawSpectrum> for MzMLReade
         match self._offset_of_index(index) {
             Some(offset) => match self.seek(SeekFrom::Start(offset)) {
                 Ok(_) => Ok(self),
-                Err(err) => Err(ScanAccessError::IOError(err)),
+                Err(err) => Err(ScanAccessError::IOError(Some(err))),
             },
             None => Err(ScanAccessError::ScanNotFound),
         }
@@ -883,14 +918,14 @@ impl<R: io::Read + io::Seek> RandomAccessScanIterator<RawSpectrum> for MzMLReade
         match self._offset_of_time(time) {
             Some(offset) => match self.seek(SeekFrom::Start(offset)) {
                 Ok(_) => Ok(self),
-                Err(err) => Err(ScanAccessError::IOError(err)),
+                Err(err) => Err(ScanAccessError::IOError(Some(err))),
             },
             None => Err(ScanAccessError::ScanNotFound),
         }
     }
 }
 
-impl<R: io::Read + io::Seek> MzMLReader<R> {
+impl<R: SeekRead> MzMLReader<R> {
     /// Construct a new MzMLReader and build an offset index
     /// using [`Self::build_index`]
     pub fn new_indexed(file: R) -> MzMLReader<R> {
@@ -961,10 +996,24 @@ impl<R: io::Read + io::Seek> MzMLReader<R> {
         self.handle
             .seek(SeekFrom::Start(start))
             .expect("Failed to restore location");
+        self.index.init = true;
+        if self.index.len() == 0 {
+            warn!("An index was built but no entries were found")
+        }
         offset
     }
 }
 
+impl MZFileReader<Spectrum> for MzMLReader<fs::File> {
+    fn open_file(source: fs::File) -> Self {
+        Self::new(source)
+    }
+
+    fn construct_index_from_stream(&mut self) -> u64 {
+        self.build_index()
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -1015,4 +1064,38 @@ mod test {
         assert_eq!(ms1_count, 14);
         assert_eq!(msn_count, 34);
     }
+
+    #[test]
+    fn reader_from_path() {
+        let path = path::Path::new("./test/data/small.mzML");
+        let mut reader = MzMLReader::open_path(path).expect("Test file doesn't exist?");
+
+        let n = reader.len();
+        assert_eq!(n, 48);
+
+        let mut ms1_count = 0;
+        let mut msn_count = 0;
+
+        for i in (0..n).rev() {
+            let scan = match reader.get_spectrum_by_index(i) {
+                Some(scan) => scan,
+                None => {
+                    if let Some(offset) = reader._offset_of_index(i) {
+                        panic!("Failed to locate spectrum {} at offset {}, parser state {:?}",
+                               i, offset, reader.state, );
+                    } else {
+                        panic!("Failed to locate spectrum or offset {}", i);
+                    }
+                }
+            };
+            let level = scan.ms_level();
+            if level == 1 {
+                ms1_count += 1;
+            } else {
+                msn_count += 1;
+            }
+        }
+        assert_eq!(ms1_count, 14);
+        assert_eq!(msn_count, 34);
+    }
 }
