@@ -1,5 +1,4 @@
-
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 pub enum MassErrorType {
     Exact,
     PPM,
