@@ -5,16 +5,22 @@ use std::ops;
 
 use crate::mass_error::MassErrorType;
 
-use super::coordinate::{CoordinateLike, IndexedCoordinate, MZ, Mass};
+use super::coordinate::{CoordinateLike, IndexedCoordinate, Mass, MZ};
 use super::peak::{CentroidPeak, DeconvolutedPeak};
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum OrderUpdateEvent {
+    TailAppend,
+    InsertResorted,
+}
+
 /// A trait for an ordered container of mass spectral peaks. The trait
 /// interoperates with [`CoordinateLike`] to make searching efficient.
 pub trait PeakCollection<T: CoordinateLike<C>, C>: ops::Index<usize>
 where
     <Self as ops::Index<usize>>::Output: CoordinateLike<C>,
 {
-    fn push(&mut self, peak: T);
+    fn push(&mut self, peak: T) -> OrderUpdateEvent;
     fn sort(&mut self);
 
     fn len(&self) -> usize;
@@ -39,6 +45,21 @@ where
             .call(self.get_item(j).get_coordinate(), query)
             .abs();
         let n = self.len();
+        // search backwards
+        while j > 0 && j < n {
+            let err = error_type
+                .call(self.get_item(j).get_coordinate(), query)
+                .abs();
+            if err < best_err && err < error_tolerance {
+                best_err = err;
+                best = j;
+            } else if err > best_err {
+                break;
+            }
+            j -= 1;
+        }
+        j = i;
+        // search forwards
         while j < n {
             let err = error_type
                 .call(self.get_item(j).get_coordinate(), query)
@@ -47,7 +68,7 @@ where
                 best_err = err;
                 best = j;
             } else if err > best_err {
-                break
+                break;
             }
             j += 1;
         }
@@ -87,14 +108,14 @@ where
             return &self.get_slice(0..0);
         }
 
-        let mut lower_index = match self._search_by(lower_bound) {
-            Ok(j) => j,
-            Err(j) => j,
+        let mut lower_index = match self.search(lower_bound, error_tolerance, error_type) {
+            Some(j) => j,
+            None => 0,
         };
 
-        let mut upper_index = match self._search_by(upper_bound) {
-            Ok(j) => j,
-            Err(j) => j,
+        let mut upper_index = match self.search(upper_bound, error_tolerance, error_type) {
+            Some(j) => j,
+            None => 0,
         };
 
         if lower_index < n {
@@ -113,6 +134,7 @@ where
         subset
     }
 
+    /// Find all peaks which could match `query` within the specified error tolerances,
     fn all_peaks_for(&self, query: f64, error_tolerance: f64, error_type: MassErrorType) -> &[T] {
         let lower_bound = error_type.lower_bound(query, error_tolerance);
         let upper_bound = error_type.upper_bound(query, error_tolerance);
@@ -122,10 +144,11 @@ where
             return &self.get_slice(0..0);
         }
 
-        let mut lower_index = match self._search_by(lower_bound) {
-            Ok(j) => j,
-            Err(j) => j,
+        let mut lower_index = match self.search(lower_bound, error_tolerance, error_type) {
+            Some(j) => j,
+            None => 0,
         };
+
         if lower_index < n {
             if self[lower_index].get_coordinate() < lower_bound {
                 lower_index += 1;
@@ -221,7 +244,7 @@ impl<P: IndexedCoordinate<C>, C> PeakCollection<P, C> for PeakSetVec<P, C> {
         Self::_sort(&mut self.peaks);
     }
 
-    fn push(&mut self, peak: P) {
+    fn push(&mut self, peak: P) -> OrderUpdateEvent {
         let n = self.len();
         match self.peaks.last() {
             Some(p) => {
@@ -229,31 +252,38 @@ impl<P: IndexedCoordinate<C>, C> PeakCollection<P, C> for PeakSetVec<P, C> {
                     self.peaks.push(peak);
                     let fin = &mut self.peaks[n];
                     fin.set_index(n as u32);
+                    OrderUpdateEvent::TailAppend
                 } else {
                     self.peaks.push(peak);
                     self.sort();
+                    OrderUpdateEvent::InsertResorted
                 }
             }
             None => {
                 self.peaks.push(peak);
                 let fin = &mut self.peaks[n];
                 fin.set_index(n as u32);
+                OrderUpdateEvent::TailAppend
             }
         }
     }
 
+    #[inline]
     fn len(&self) -> usize {
         self.peaks.len()
     }
 
+    #[inline]
     fn get_item(&self, i: usize) -> &P {
         &self[i]
     }
 
+    #[inline]
     fn get_slice(&self, i: ops::Range<usize>) -> &[P] {
         &self.peaks[i]
     }
 
+    #[inline]
     fn _search_by(&self, query: f64) -> Result<usize, usize> {
         self.peaks
             .binary_search_by(|peak| peak.get_coordinate().partial_cmp(&query).unwrap())
@@ -412,7 +442,7 @@ mod test {
         let mut reader =
             MGFReader::new(fs::File::open("./test/data/small.mgf").expect("Missing test file"));
         let scan = reader.next().expect("Failed to read first spectrum");
-        let peaks = scan.peaks;
+        let peaks = scan.into_centroid().unwrap().peaks;
 
         assert_eq!(peaks.len(), 485);
         assert!((peaks[0].mz - 231.3888).abs() < 1e-3);
