@@ -13,14 +13,14 @@
 //!
 //! These structures all implement the [`SpectrumBehavior`] trait
 use std::borrow;
-
-use crate::peaks::CentroidPeak;
-use crate::peaks::{PeakCollection, PeakSet};
+use std::convert::TryFrom;
+use crate::mass_error::MassErrorType;
+use crate::peaks::{CentroidPeak};
+use crate::peaks::{PeakCollection, PeakSet, DeconvolutedPeakSet};
 use crate::spectrum::scan_properties::{
     Acquisition, Precursor, SignalContinuity, SpectrumDescription,
 };
-use crate::spectrum::signal::BinaryArrayMap;
-use crate::mass_error::MassErrorType;
+use crate::spectrum::signal::{BinaryArrayMap, ArrayType};
 
 #[derive(Debug)]
 /// An variant for dispatching to different strategies of computing
@@ -29,56 +29,71 @@ pub enum PeakDataLevel<'lifespan> {
     Missing,
     RawData(&'lifespan BinaryArrayMap),
     Centroid(&'lifespan PeakSet),
+    Deconvoluted(&'lifespan DeconvolutedPeakSet),
 }
 
 impl<'lifespan> PeakDataLevel<'lifespan> {
     /// Compute the base peak of a spectrum
     pub fn base_peak(&self) -> (usize, f64, f32) {
         match self {
-            PeakDataLevel::Missing => {
-                (0, 0.0, 0.0)
-            },
+            PeakDataLevel::Missing => (0, 0.0, 0.0),
             PeakDataLevel::RawData(arrays) => {
                 let intensities = arrays.intensities();
-                let result = intensities.iter().enumerate().max_by(|ia, ib| ia.1.partial_cmp(&ib.1).unwrap());
+                let result = intensities
+                    .iter()
+                    .enumerate()
+                    .max_by(|ia, ib| ia.1.partial_cmp(&ib.1).unwrap());
                 if let Some((i, inten)) = result {
                     (i, arrays.mzs()[i], *inten)
                 } else {
                     (0, 0.0, 0.0)
                 }
-            },
+            }
             PeakDataLevel::Centroid(peaks) => {
-                let result = peaks.iter().enumerate().max_by(|ia, ib| ia.1.intensity.partial_cmp(&ib.1.intensity).unwrap());
+                let result = peaks
+                    .iter()
+                    .enumerate()
+                    .max_by(|ia, ib| ia.1.intensity.partial_cmp(&ib.1.intensity).unwrap());
                 if let Some((i, peak)) = result {
                     (i, peak.mz, peak.intensity)
                 } else {
                     (0, 0.0, 0.0)
                 }
+            },
+            PeakDataLevel::Deconvoluted(peaks) => {
+                let result = peaks
+                    .iter()
+                    .enumerate()
+                    .max_by(|ia, ib| ia.1.intensity.partial_cmp(&ib.1.intensity).unwrap());
+                if let Some((i, peak)) = result {
+                    (i, peak.neutral_mass, peak.intensity)
+                } else {
+                    (0, 0.0, 0.0)
+                }
             }
         }
     }
 
     pub fn tic(&self) -> f32 {
         match self {
-            PeakDataLevel::Missing => {
-                0.0
-            },
+            PeakDataLevel::Missing => 0.0,
             PeakDataLevel::RawData(arrays) => {
                 let intensities = arrays.intensities();
                 intensities.iter().sum()
-            },
-            PeakDataLevel::Centroid(peaks) => {
-                peaks.iter().map(|p|p.intensity).sum()
             }
+            PeakDataLevel::Centroid(peaks) => peaks.iter().map(|p| p.intensity).sum(),
+            PeakDataLevel::Deconvoluted(peaks) => peaks.iter().map(|p| p.intensity).sum()
         }
     }
 
-    pub fn search(&self, query: f64, error_tolerance: f64, error_type: MassErrorType) -> Option<usize> {
-
+    pub fn search(
+        &self,
+        query: f64,
+        error_tolerance: f64,
+        error_type: MassErrorType,
+    ) -> Option<usize> {
         match self {
-            PeakDataLevel::Missing => {
-                None
-            },
+            PeakDataLevel::Missing => None,
             PeakDataLevel::RawData(arrays) => {
                 let mzs = arrays.mzs();
                 let lower = error_type.lower_bound(query, error_tolerance);
@@ -96,16 +111,15 @@ impl<'lifespan> PeakDataLevel<'lifespan> {
                             index += 1;
                         }
                         if best_error < error_tolerance {
-                            return Some(best_index)
+                            return Some(best_index);
                         }
                         None
-                    },
-                    Err(_err) => None
+                    }
+                    Err(_err) => None,
                 }
-            },
-            PeakDataLevel::Centroid(peaks) => {
-                peaks.search(query, error_tolerance, error_type)
             }
+            PeakDataLevel::Centroid(peaks) => peaks.search(query, error_tolerance, error_type),
+            PeakDataLevel::Deconvoluted(peaks) => peaks.search(query, error_tolerance, error_type)
         }
     }
 }
@@ -181,6 +195,7 @@ pub struct RawSpectrum {
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub enum SpectrumConversionError {
     MZIntensityArraySizeMismatch,
+    NotDeconvoluted,
     NotCentroided,
     NoPeakData,
 }
@@ -199,7 +214,7 @@ impl<'transient, 'lifespan: 'transient> RawSpectrum {
         let intensity_array = self.intensities();
 
         if mz_array.len() != intensity_array.len() {
-            return Err(SpectrumConversionError::NotCentroided);
+            return Err(SpectrumConversionError::MZIntensityArraySizeMismatch);
         }
 
         let mut peaks = PeakSet::empty();
@@ -277,6 +292,24 @@ impl CentroidSpectrum {
     }
 }
 
+#[derive(Default, Debug, Clone)]
+struct DeconvolutedSpectrum {
+    /// The spectrum metadata describing acquisition conditions and details.
+    pub description: SpectrumDescription,
+    /// The deisotoped and charge state deconvolved peaks
+    pub deconvoluted_peaks: DeconvolutedPeakSet,
+}
+
+impl<'lifespan> SpectrumBehavior for DeconvolutedSpectrum {
+    fn description(&self) -> &SpectrumDescription {
+        &self.description
+    }
+
+    fn peaks(&'_ self) -> PeakDataLevel<'_> {
+        PeakDataLevel::Deconvoluted(&self.deconvoluted_peaks)
+    }
+}
+
 #[derive(Default, Debug, Clone)]
 /// Represent a spectrum with multiple layers of representation of the
 /// peak data.
@@ -306,7 +339,6 @@ impl<'lifespan> SpectrumBehavior for Spectrum {
             PeakDataLevel::Missing
         }
     }
-
 }
 
 impl Spectrum {
@@ -319,14 +351,25 @@ impl Spectrum {
             };
             result.description.signal_continuity = SignalContinuity::Centroid;
             return Ok(result);
-        } else if let Some(arrays) = &self.arrays {
-            let peaks = PeakSet::from(arrays);
-            let mut centroid = CentroidSpectrum {
-                description: self.description,
-                peaks,
-            };
-            centroid.description.signal_continuity = SignalContinuity::Centroid;
-            return Ok(centroid);
+        } else {
+            if self.signal_continuity() == SignalContinuity::Centroid {
+                if let Some(arrays) = &self.arrays {
+                    let peaks = PeakSet::from(arrays);
+                    let mut centroid = CentroidSpectrum {
+                        description: self.description,
+                        peaks,
+                    };
+                    centroid.description.signal_continuity = SignalContinuity::Centroid;
+                    return Ok(centroid);
+                } else {
+                    let mut result = CentroidSpectrum {
+                        peaks: PeakSet::empty(),
+                        description: self.description,
+                    };
+                    result.description.signal_continuity = SignalContinuity::Centroid;
+                    return Ok(result);
+                }
+            }
         }
         Err(SpectrumConversionError::NotCentroided)
     }
@@ -348,7 +391,62 @@ impl Spectrum {
             result.description.signal_continuity = SignalContinuity::Centroid;
             Ok(result)
         } else {
-            Err(SpectrumConversionError::NoPeakData)
+            let arrays = BinaryArrayMap::from(PeakSet::empty());
+            Ok(RawSpectrum {
+                arrays,
+                description: self.description,
+            })
         }
     }
 }
+
+impl From<Spectrum> for CentroidSpectrum {
+    fn from(spectrum: Spectrum) -> CentroidSpectrum {
+        spectrum.into_centroid().unwrap()
+    }
+}
+
+impl From<CentroidSpectrum> for Spectrum {
+    fn from(spectrum: CentroidSpectrum) -> Spectrum {
+        spectrum.into_spectrum().unwrap()
+    }
+}
+
+impl From<RawSpectrum> for Spectrum {
+    fn from(spectrum: RawSpectrum) -> Spectrum {
+        spectrum.into_spectrum().unwrap()
+    }
+}
+
+impl From<Spectrum> for RawSpectrum {
+    fn from(spectrum: Spectrum) -> RawSpectrum {
+        spectrum.into_raw().unwrap()
+    }
+}
+
+impl From<RawSpectrum> for CentroidSpectrum {
+    fn from(spectrum: RawSpectrum) -> CentroidSpectrum {
+        spectrum.into_centroid().unwrap()
+    }
+}
+
+impl TryFrom<Spectrum> for DeconvolutedSpectrum {
+    type Error = SpectrumConversionError;
+
+    fn try_from(spectrum: Spectrum) -> Result<Self, Self::Error> {
+        if spectrum.signal_continuity() == SignalContinuity::Profile {
+            return Err(SpectrumConversionError::NotCentroided);
+        }
+        if let Some(arrays) = &spectrum.arrays {
+            if arrays.has_array(&ArrayType::ChargeArray) {
+                let peaks: DeconvolutedPeakSet = DeconvolutedPeakSet::from(arrays);
+                return Ok(DeconvolutedSpectrum {
+                    description: spectrum.description,
+                    deconvoluted_peaks: peaks
+                })
+            }
+            return Err(Self::Error::NotDeconvoluted)
+        }
+        return Err(Self::Error::NoPeakData)
+    }
+}
\ No newline at end of file
